#ifndef H_CANNON_WEAPON_SLOT
#define H_CANNON_WEAPON_SLOT
#include "WeaponSlot.h"

class CannonWeaponSlot: public WeaponSlot{
    public:
        /**
          return reloading time of weapon.
          reloading time is for 1 shoot.
          note that cannonBall counts as more shoots
          */
        virtual double getReloadTime();

        /**
          return minimal time to wait, until next shoot
          is allowed
          */
        virtual double getTimeToNextShoot();

        /**
          return actual number of shoot that can be shoot,
          note that this number>0 do not mean that you can shoot
          - also getTimeToNextShoot() must be zero!
          */
        virtual double getNumberOfRemainingShoots();

        /**
          return maximum energy of WeaponLoaders,
          */
        virtual double getMaxEnergy();

        /**
          return energy needed to shoot
          */
        virtual double getShootEnergy();

        /**
          return actual state of weapon energy.
          note that it is incremented by 1energy/second
          */
        virtual double getActualEnergy();

        /**
          get minimum delta angle to shoot
          one can shoot in range (getMinAngle()+actualAngle,
          getMaxAngle()+actualAngle)
          */
        virtual double getMinAngle();

        /**
          @see getMinAngle()
          get maximum delta angle to shoot
          */
        virtual double getMaxAngle();

        /**
          get minimal speed of projectile upon shooting
          note that this value is relative to ship's speed
          */
        virtual double getMinSpeed();

        /**          
          get maximal speed of projectile upon shooting
          note that this value is relative to ship's speed
          */
        virtual double getMaxSpeed();

        /**
          initialize all static variables
          */
        static void initialize();

        /**
          destructor
          */
        virtual ~CannonWeaponSlot();

        /**
          constructor, clears all internal variables
          */
        CannonWeaponSlot();

        /**
          notify slot that there is new time quantum
          */
        virtual void timeNotify(double timeDelta);

        /**
          check if ship can shoot and if can, shoot
          */
        virtual void shoot(Ship* ship, double angle, double speed,
                 double additionalInfo, World* world);

        /**
          clone slot, internal use by ship's clone
          */
        virtual WeaponSlot* clone();

        /**
          convert to bytestream
          */
        virtual void serialize(std::vector<unsigned char>& destination);

        /**
          convert bytestream to CannonWeaponSlot, note that marker
          must be removed
          */
        static CannonWeaponSlot* deserialize(std::vector<unsigned
            char>::iterator& it);

    private:
        /**
          actual reload time
          */
        double reload;

        /**
          time to reload after shoot
          */
        static double reloadTime;

        /**
          minimum count of cannon balls
          */
        static int minCount;

        /**
          maximum count of cannon balls
          */
        static int maxCount;

        /**
          internal variable - angle to scatter cannon balls
          */
        static double scatterAngle;

        /**
          speed of one cannon ball
          */
        static double cannonBallSpeed;

        /**
          internal variable, distance to spawn cannonBall from
          center of ship
          */
        static double shootDistance;

        /**
          internal variable, factor of throwing shooter back
          */
        static double recoilFactor;

        /**
          internal variable, variability in spawning distance,
          see @shootDistance
          */
        static double distanceVariablity;
};

#endif
