#ifndef H_COMMUNICATION
#define H_COMMUNICATION

#include <vector>
#include <functional>
#include <string>

#include "CommunicationGraph.h"
class Communication;
#include "World.h"
#include "PlayerResponse.h"
#include "Config.h"
class PlayerStatistics;

class CommunicationShip{
        int player,ships;
	int fd,sfd,pid,packet_len;
	int state;
	std::vector<unsigned char> data_v;
	int data_send_len;
	int data_receive_len;
	std::string filename;
	std::vector<PlayerResponse*> response;
//	World *world;
	Config *config;
	bool active;
	int connectWithPlayer(); 
	int trySend();
	int tryReceive();
public:
	CommunicationShip(int player,int ships,Config *config);
	~CommunicationShip();
//	PlayerResponse* communicationFoo(World* world);
	int send(std::vector<unsigned char> &data);
	PlayerResponse* receive(int ship);
	int connected();
};

class CommunicationViewer{
	int fd;
	bool sending;
	std::vector<unsigned char>data_v;
	int data_send_len;
	int pid;
public:
	CommunicationViewer(int fd);
	CommunicationViewer(int fd,int pid);
	~CommunicationViewer();
	int SendData(std::vector<unsigned char>&data);
};

class Communication 
{
private:
        std::vector<CommunicationShip*> ships;	
	Config *config;
	std::vector<unsigned char> data,data_stat;
	World *world;
	std::vector<CommunicationViewer*> remoteView;
	std::vector<CommunicationViewer*> localView;
	std::vector<CommunicationViewer*> localStats;
	std::vector<CommunicationViewer*> remoteStats;
	std::vector<std::pair<unsigned char,unsigned char> > kill;//killer,victim
	std::vector<int> event;
	int remotefd,remotefdstat;
public:
	Communication ();
	~Communication();
	void initializeServer();
	void initializeClient();
	int playersConnected();
	int sendNewGame (CommunicationGraph* comm,
	                 std::vector<int> ships);
	int sendPlayerData(World* world);
	PlayerResponse* receivePlayerResponse (int player,int ship);
	int sendDataToViewer (World* world);
        int sendStatisticsData (PlayerStatistics* stats);


        // events
        void shipKilledShip(int killer, int victim); //
        void blackHoleKilledShip(int victim); //
        void impactKilledShip(int victim); //
        void explosionKilledShip(int victim); //
        void shootPlasma();
        void shootMissile();
        void shootCannon();
        void explode(); //
        void plasmaToPlasmaExplode(); //
};

#endif
