#ifndef H_GRAVITY
#define H_GRAVITY

#include "MassObject.h"
#include "Point.h"
#include <vector>

class Gravity{
    public:
        /**
          constructor
          */
        Gravity();

        /**
          initializes gravity, read preferences
          */
        void initialize();

        /**
          clear heavy object's cache
          */
        void clear();

        /**
          insert heavy objects into heavy object's cache
          */
        // {{{ setGravitySources
        template <typename InputIterator>
            void setGravitySources(InputIterator begin,
                    InputIterator end){
                while (begin!=end){
                    position.push_back((*begin)->position);
                    mass.push_back((*begin)->mass);
                    radius.push_back((*begin)->radius);
                    ++begin;
                }
        }
        // }}}

        /**
          apply gravity to objects
          */
        // {{{ applyGravityTo
        template <typename InputIterator>
            void applyGravityTo(InputIterator begin,
                    InputIterator end){
                while (begin!=end){
                    applyGravityToObject(*begin);
                    ++begin;
                }
        }
        // }}}



    private:
        /**
          position cache
          */
        std::vector<Point>  position;

        /**
          mass cache
          */
        std::vector<double> mass;

        /**
          radius cache
          */
        std::vector<double> radius;

        /**
          gravitational constant
          */
        double gravitationalKappa;

        /**
          apply gravity to one object
          */
        void applyGravityToObject(MassObject* object);
};


#endif
