#ifndef H_WEAPON_SLOT
#define H_WEAPON_SLOT

#include <vector>

class Ship;
class World;

class WeaponSlot{
    public:
	virtual ~WeaponSlot();
        /**
          return reloading time of weapon.
          reloading time is for 1 shoot.
          note that cannonBall counts as more shoots
          */
        virtual double getReloadTime()=0;

        /**
          return minimal time to wait, until next shoot
          is allowed
          */
        virtual double getTimeToNextShoot()=0;

        /**
          return actual number of shoot that can be shoot,
          note that this number>0 do not mean that you can shoot
          - also getTimeToNextShoot() must be zero!
          */
        virtual double getNumberOfRemainingShoots()=0;

        /**
          return maximum energy of WeaponLoaders,
          */
        virtual double getMaxEnergy()=0;

        /**
          return energy needed to shoot
          */
        virtual double getShootEnergy()=0;

        /**
          return actual state of weapon energy.
          note that it is incremented by 1energy/second
          */
        virtual double getActualEnergy()=0;

        /**
          get minimum delta angle to shoot
          one can shoot in range (getMinAngle()+actualAngle,
          getMaxAngle()+actualAngle)
          */
        virtual double getMinAngle()=0;

        /**
          @see getMinAngle()
          get maximum delta angle to shoot
          */
        virtual double getMaxAngle()=0;

        /**
          get minimal speed of projectile upon shooting
          note that this value is relative to ship's speed
          */
        virtual double getMinSpeed()=0;

        /**          
          get maximal speed of projectile upon shooting
          note that this value is relative to ship's speed
          */
        virtual double getMaxSpeed()=0;

    public:
        /**
          notify WeaponSlot of new time quantum
          */
        virtual void timeNotify(double timeDelta)=0;

        /**
          player wants to shoot - shoot if possible
          */
        virtual void shoot(Ship* ship, double angle, double speed,
                double additionalInfo, World* world)=0;

        /**
          make copy of weaponSlot, used internally by ship's copy
          */
        virtual WeaponSlot* clone()=0;

        /**
          convert WeaponSlot to bytestream
          */
        virtual void serialize(std::vector<unsigned char>&
                destination)=0;

        /**
          convert bytestream to specific WeaponSlot
          */
        static WeaponSlot* _deserialize(std::vector<unsigned
                char>::iterator&
                it);

};

#endif
