
#define RETREAT 0
#define ATTACK 1
#define FUN 2
#define HMLOVINA 3

#define MAX_SHIPS 100
#define MAX_DIST_ERROR 10
#define MAX_ANGLE_ERROR 0.3
#define ODVAHA 15
#define SUKROMIE 20
#define BOJI_SA_STRIEL true
#define BOJI_SA_SVOJICH true
#define CIRCLE_FORMATION_RADIUS 300
#define STRATEGY_HMLOVINA false

#include "player.h"
#include<iostream>
#include<string>
#include<queue>
#include<map>

using namespace std;


/********* CLASSES INTERFACE *********/

class Intruder{
public:
  Intruder();
  ~Intruder();
  bool operator< (const Intruder& i);
  bool operator> (const Intruder& i);
  int evilID;
  MassObject* pThis;
  int emergency;
};

Intruder::Intruder(){
}

struct Cmp{
  bool operator()(const Intruder a,const Intruder b)const{
    return a.emergency<b.emergency;
  }
};

class Intruders{
  public:
    Intruders();
    ~Intruders();
    priority_queue<Intruder,vector<Intruder>,Cmp> worst;
    int emergency;
    void push(Intruder i);
    void pushID(int ID);
};

Intruders::~Intruders(){
}

/******** END CLASSES INTERFACE **********/

/******** GLOBAL VARIABLES ***********/
Intruders intruders;
Intruders victims;
int strategy[MAX_SHIPS];
int lastStrategy[MAX_SHIPS];
unsigned int tick;
bool alert;
MassObject* currentShip;
WorldArrays* currentWorld;
Point formationCenter;
double phi[MAX_SHIPS];
Point target[MAX_SHIPS];

/********* END GLOBAL VARIABLES *********/

/******* GET METHODS ********/

BlackHole* getBlackHoleById(int id){
  FOREACH(i,currentWorld->blackHoles){
    if((*i).id==id)return &*i;
  }
}

Asteroid* getAsteroidById(int id){
  FOREACH(i,currentWorld->asteroids){
    if((*i).id==id)return &*i;
  }
}

Twirler* getTwirlerById(int id){
  FOREACH(i,currentWorld->twirlers){
    if((*i).id==id)return &*i;
  }
}

Nebula* getNebulaById(int id){
  FOREACH(i,currentWorld->nebulas){
    if((*i).id==id)return &*i;
  }
}

Ship* getShipByID(int id){
  FOREACH(i,currentWorld->myShips){
    if((*i).id==id)return &*i;
  }
  FOREACH(i,currentWorld->otherShips){
    if((*i).id==id)return &*i;
  }
}

PlasmaBall* getPlasmaBallByID(int id){
  FOREACH(i,currentWorld->plasmaBalls){
    if((*i).id==id)return &*i;
  }
}

CannonBall* getCannonBallByID(int id){
  FOREACH(i,currentWorld->cannonBalls){
    if((*i).id==id)return &*i;
  }
}

Missile* getMissileByID(int id){
  FOREACH(i,currentWorld->missile){
    if((*i).id==id)return &*i;
  }
}

MassObject* getObjectByID(int id){
  FOREACH(i,currentWorld->blackHoles){
    if((*i).id==id)return &*i;
  }
  FOREACH(i,currentWorld->asteroids){
    if((*i).id==id)return &*i;
  }
/*  FOREACH(i,currentWorld->twirlers){
    if((*i).id==id)return &*i;
  }
  FOREACH(i,currentWorld->nebulas){
    if((*i).id==id)return &*i;
  }
*/  FOREACH(i,currentWorld->myShips){
    if((*i).id==id)return &*i;
  }
  FOREACH(i,currentWorld->otherShips){
    if((*i).id==id)return &*i;
  }
  FOREACH(i,currentWorld->plasmaBalls){
    if((*i).id==id)return &*i;
  }
  FOREACH(i,currentWorld->cannonBalls){
    if((*i).id==id)return &*i;
  }
  FOREACH(i,currentWorld->missile){
    if((*i).id==id)return &*i;
  }


}

string getTypeByID(int id){
  FOREACH(i,currentWorld->blackHoles){
    if((*i).id==id)return "blackHole";
  }
  FOREACH(i,currentWorld->asteroids){
    if((*i).id==id)return "asteroid";
  }
  FOREACH(i,currentWorld->twirlers){
    if((*i).id==id)return "twirler";
  }
  FOREACH(i,currentWorld->nebulas){
    if((*i).id==id)return "nebula";
  }
  FOREACH(i,currentWorld->myShips){
    if((*i).id==id)return "myShip";
  }
  FOREACH(i,currentWorld->otherShips){
    if((*i).id==id)return "otherShip";
  }
  FOREACH(i,currentWorld->plasmaBalls){
    if((*i).id==id)return "plasmaBall";
  }
  FOREACH(i,currentWorld->cannonBalls){
    if((*i).id==id)return "cannonBall";
  }
  FOREACH(i,currentWorld->missile){
    if((*i).id==id)return "missile";
  }

  return "none";
}

int getFearByType(string type){
  if(type=="plasma")return 1300;
  if(type=="blackHole")return 3000;
  if(type=="myShip")return 600;
  if(type=="otherShip")return 1400;
  return 900;
}

int getDistanceByID(int id){
  string type=getTypeByID(id);
  if(type=="none")return 100000;
  if(type=="blackHole")return Utils::distance(getBlackHoleById(id),currentShip);
  if(type=="asteroid")return Utils::distance(getAsteroidById(id),currentShip);       
  if(type=="twirler")return Utils::distance(getAsteroidById(id),currentShip);
  if(type=="nebula")return Utils::distance(getAsteroidById(id),currentShip);
  if(type=="otherShip" || type=="myShip")return Utils::distance(getShipByID(id),currentShip);
  if(type=="plasmaBall")return Utils::distance(getPlasmaBallByID(id),currentShip);
  if(type=="cannonBall")return Utils::distance(getCannonBallByID(id),currentShip);
  if(type=="missile")return Utils::distance(getCannonBallByID(id),currentShip);
}

/****** END GET METHODS ********/

/********** CLASSES IMPLEMENTATION *********/

Intruder::~Intruder(){
}

bool Intruder::operator< (const Intruder& i){
  return emergency<i.emergency;
}

bool Intruder::operator> (const Intruder& i){
  return emergency>i.emergency;
}


Intruders::Intruders(){
  emergency=0;
}

void Intruders::push(Intruder i){
  worst.push(i);
  emergency=worst.top().emergency;
}

void Intruders::pushID(int ID){
  if(ID==currentShip->id)return;
  Intruder in=Intruder();
  in.evilID=ID;
  in.pThis=getObjectByID(ID);
  //cout << currentShip->position.x << ":" << currentShip->position.y << "\n" << currentShip->velocity.x << ":" << currentShip->velocity.y << "\n";
  //cout << in.pThis->position.x << ":" << in.pThis->position.y << "\n" << in.pThis->velocity.x << ":" << in.pThis->velocity.y << "\n";
  string type = getTypeByID(ID);
  in.emergency=getFearByType(type)/(0.5+Utils::getTimeToDistance(currentShip,in.pThis,SUKROMIE));
  //cout << in.emergency << endl;
  worst.push(in);
  emergency=worst.top().emergency;
}

/******** END CLASSES IMPLEMENTATION **********/

void randomFire(PlayerResponse* playerResponse){
   if(rand()%2){
     playerResponse->fireWeapon=1;
     playerResponse->weaponSpeed=getShipByID(currentShip->id)->weaponSlots[1]->getMaxSpeed();
     playerResponse->weaponAngle=0;
     playerResponse->weaponAdditional=6;
   }else{
     playerResponse->fireWeapon=(rand()%2)*2;
     playerResponse->weaponSpeed=getShipByID(currentShip->id)->weaponSlots[0]->getMaxSpeed();
     playerResponse->weaponAngle=0;
     playerResponse->weaponAdditional=0;
   }

}

bool somVHmlovine(Nebula* nebula){
  return (Utils::distance(currentShip,nebula) < nebula->radius-MAX_DIST_ERROR);
}

bool somVNejakejHmlovine(){
  bool som=false;
  FOREACH(i,currentWorld->nebulas)
    som|=somVHmlovine(&*i);
  return som;
}

int najblizsiaHmlovinaID(){
  double dist=9999999;
  Nebula* n;
  FOREACH(i,currentWorld->nebulas){
    double temp=Utils::distance(currentShip,&*i);
    if(temp<dist){
      dist=temp;
      n=&*i;
    }
  }
  return n->id;
}

void emergency(){
  intruders=Intruders();
  FOREACH(i,currentWorld->blackHoles)
    intruders.pushID((*i).id);
  FOREACH(i,currentWorld->asteroids)
    intruders.pushID((*i).id);
/*  FOREACH(i,currentWorld->twirlers)
    intruders.pushID((*i).id);
  FOREACH(i,currentWorld->nebulas)
    intruders.pushID((*i).id);
*/if(BOJI_SA_SVOJICH){
  FOREACH(i,currentWorld->myShips)
    intruders.pushID((*i).id);
  }
  FOREACH(i,currentWorld->otherShips)
    intruders.pushID((*i).id);
  if(BOJI_SA_STRIEL){
  FOREACH(i,currentWorld->plasmaBalls)
    intruders.pushID((*i).id);
  FOREACH(i,currentWorld->cannonBalls)
    intruders.pushID((*i).id);
  FOREACH(i,currentWorld->missile)
    intruders.pushID((*i).id);
  }
  if(intruders.emergency>ODVAHA)alert=true;

}

void findVictim(){
  victims=Intruders();
  FOREACH(i,currentWorld->otherShips)
    victims.pushID((*i).id);
}

void init_game(){
  intruders=Intruders();
  for(int i=0;i<MAX_SHIPS;i++){
    lastStrategy[i]=strategy[i]=ATTACK;
  }
  tick=0;
  formationCenter=Point(0,0);
}


void think(WorldArrays &world,std::vector<PlayerResponse> &response){
    currentWorld=&world;
    //bool willBeFun = true;
    //bool newFun = true;
    if(STRATEGY_HMLOVINA){
      for(unsigned int i=0;i<response.size();i++)
        strategy[i]=HMLOVINA;
    }
    //for(unsigned int  i=0;i<response.size();i++){
    //  newFun &= (lastStrategy[i] != FUN);
    //  willBeFun &= (lastStrategy[i] != RETREAT);
    //  phi[i] = 2*M_PI*i/response.size();
    //}
    //if(newFun && willBeFun){
    //  cout << "Bude formacia\n";
    //  formationCenter=Point(0,0);
    //  for(unsigned int  i=0;i<response.size();i++){
    //   formationCenter+=currentWorld->myShips.at(i).position;
    //  }
    //  formationCenter/=(double)response.size();
    //  for(unsigned int  i=0;i<response.size();i++){
    //    target[i]=formationCenter+Point(CIRCLE_FORMATION_RADIUS*cos(phi[i]),CIRCLE_FORMATION_RADIUS*sin(phi[i]));
    //  }
    //  
    //}

    for(unsigned int i=0;i<response.size();i++){
      tick++;
      PlayerResponse &playerResponse=response[i];
      currentShip=getObjectByID(currentWorld->myShips.at(i).id);
      alert=false;
      emergency();
      if(alert){
        strategy[i]=RETREAT;
    //    if(willBeFun)cout << "Konci zabava, nastava tvrdy boj\n";
    //    willBeFun=false;
    //  }
    //  else if(willBeFun){
    //    strategy[i]=FUN;
      }else{
        if(STRATEGY_HMLOVINA)
          strategy[i]=HMLOVINA;
        else strategy[i]=ATTACK;
      }
      /******** R E T R E A T **********/

      if(strategy[i]==RETREAT){
        double intruderAngle = Utils::getAngle(currentShip,intruders.worst.top().pThis);
        double deltaAngle = Utils::normalizeAngle(intruderAngle - currentShip->angle);

        if(abs(deltaAngle)<M_PI/2){
          playerResponse.accelerationJets=-1;
          playerResponse.rotationJets=
            Utils::getShipRotationalJets(&currentWorld->myShips.at(i),intruderAngle);
          string type = getTypeByID(intruders.worst.top().evilID);
          if(type!="myShip" && type!="plasmaBall"){
            if(rand()%2==0){
            playerResponse.fireWeapon=1;
            playerResponse.weaponSpeed=getShipByID(currentShip->id)->weaponSlots[1]->getMaxSpeed();
            playerResponse.weaponAngle=0;
            playerResponse.weaponAdditional=6;
            }else{
              playerResponse.fireWeapon=2;
              playerResponse.weaponSpeed=getShipByID(currentShip->id)->weaponSlots[2]->getMaxSpeed();
              playerResponse.weaponAngle=0;
              playerResponse.weaponAdditional=0;
            }
          }else{
            //do svojho ani do plazmy nebudem strielat
            playerResponse.fireWeapon=-1;
          }
        }else{
          //pridavaj
          playerResponse.accelerationJets=1;
          playerResponse.rotationJets=
            Utils::getShipRotationalJets(&currentWorld->myShips.at(i),intruderAngle+M_PI);
          playerResponse.fireWeapon=-1;
        }
      }//end if RETREAT

      /********** A T T A C K **********/

      if(strategy[i]==ATTACK){
        findVictim();
        double victimAngle = Utils::getAngle(currentShip,victims.worst.top().pThis);
        playerResponse.rotationJets=
          Utils::getShipRotationalJets(&currentWorld->myShips.at(i),victimAngle);
	playerResponse.accelerationJets=(rand()%2);
         //{
         //  playerResponse.fireWeapon=0;
         //  playerResponse.weaponSpeed=50;
         //  playerResponse.weaponAngle=0;
         //  playerResponse.weaponAdditional=0;
	 //}else{
         if(Utils::normalizeAngle(victimAngle-currentShip->angle)<MAX_ANGLE_ERROR){
           if(rand()%2==0){
           playerResponse.fireWeapon=1;
           playerResponse.weaponSpeed=getShipByID(currentShip->id)->weaponSlots[1]->getMaxSpeed();
           playerResponse.weaponAngle=0;
           playerResponse.weaponAdditional=6;
           }else{
             playerResponse.fireWeapon=2;
             playerResponse.weaponSpeed=getShipByID(currentShip->id)->weaponSlots[2]->getMaxSpeed();
             playerResponse.weaponAngle=0;
             playerResponse.weaponAdditional=0;
           }
         }
         //}
      }//end if ATTACK

      //if(strategy[i]==FUN){
        //target[i]=Point(400,400);
      //  Utils::shipNavigate(getShipByID(currentShip->id),&playerResponse,target[i]);
      //  playerResponse.fireWeapon=-1;
      //  double targetAngle = Utils::getAngle(currentShip->position,target[i]);
      //  int maDobrySmer = currentShip->velocity*(target[i]-currentShip->position)>0?1:-1;//1 ak ano, -1 ak nie

      //  playerResponse.accelerationJets=
      //    Utils::getShipAccelerationJets(&currentWorld->myShips.at(i),target[i]-currentShip->position);
        //targetAngle = Utils::getObjectToTargetAngleWithRespectToVelocity(currentShip,target[i],playerResponse.accelerationJets>0?1:0);
      //  playerResponse.rotationJets=
      //    Utils::getShipRotationalJets(&currentWorld->myShips.at(i),targetAngle);
        /*if((currentShip->position-target[i]).length()>MAX_DIST_ERROR){
          playerResponse.rotationJets=
            Utils::getShipRotationalJets(&currentWorld->myShips.at(i),targetAngle);
          if(Utils::normalizeAngle(targetAngle-currentShip->angle)<MAX_ANGLE_ERROR)
            playerResponse.accelerationJets=
              Utils::getShipAccelerationJets(&currentWorld->myShips.at(i),(currentShip->position-target[i]).length());
        }else{
          //is in formation
        }*/
      //}//end if FUN

    if(strategy[i]==HMLOVINA){
      Nebula* nebula = getNebulaById(najblizsiaHmlovinaID());
      Utils::shipNavigate(getShipByID(currentShip->id),&playerResponse,nebula->position);
      if(somVNejakejHmlovine()){
        if(abs(Utils::getAngle(currentShip,nebula))>M_PI*3/4){
          cout << "salatim\n";
          randomFire(&playerResponse);
          playerResponse.accelerationJets=0;
          playerResponse.rotationJets=0;
        }
      }else{
        playerResponse.fireWeapon=-1;
      }
    }//end if HMLOVINA
    //if(lastStrategy[i]!=strategy[i])cout << strategy[i] << endl;
    lastStrategy[i]=strategy[i];
    }
}
