#include "Asteroid.h"
#include "Serializer.h"
#include "Types.h"
#include "World.h"
#include "Preferences.h"
#include "Expression.h"
#include "ObjectGraphics.h"

double Asteroid::minSpawnRadius;
double Asteroid::density;
double Asteroid::radiusToLive;
double Asteroid::splitRadius;
double Asteroid::splitVelocityFactor;

// {{{ getType OK
int Asteroid::getType(){
    return Types::asteroid;
}
// }}}

// {{{ serialize OK
void Asteroid::serialize(std::vector<unsigned char>& destination){
    Serializer::serializeChar(destination,getType());
    BasicObject::_serialize(destination,this);
    Serializer::serializeFloat(destination,live);
}
// }}}

// {{{ deserialize OK
Asteroid* Asteroid::deserialize(std::vector<unsigned char>::iterator&
        it) {
    Asteroid* ptr=new Asteroid();
    BasicObject::_deserialize(it,ptr);
    ptr->live=Serializer::deserializeFloat(it);
    return ptr;
}

// }}}

// {{{ Asteroid OK
Asteroid::Asteroid():MassObject(){
    live=1;
}
//}}}

// {{{ ~Asteroid OK
Asteroid::~Asteroid(){

}
// }}}


// {{{ doDamage OK
void Asteroid::doDamage(double impactDamage, double explosionDamage,
        int from, World* world){
    MassObject::doDamage(impactDamage,explosionDamage,from,world);
    live-=impactDamage+explosionDamage;
    if (live<0) alive=0;
}
// }}}

// {{{ setRadius OK
void Asteroid::setRadius(double radius){
    this->radius=radius;
    mass=3.14*radius*radius*density;
    angularMass=mass*radius*radius/2;
    live=radius*radiusToLive;
}
// }}}

// {{{ destroy OK
void Asteroid::destroy(World* world){
    double a=Random::getDouble(M_PI);
    Point smer=Point(cos(a),sin(a));
    double newSize=radius/splitRadius;
    if (newSize<minSpawnRadius) return;

    Asteroid* ptr1=new Asteroid();
    ptr1->position=this->position+smer*newSize*1.1;
    ptr1->velocity=this->velocity+smer*this->velocity.length()*splitVelocityFactor;
    ptr1->setRadius(newSize);
    world->addLightObject(ptr1);

    Asteroid* ptr2=new Asteroid();
    ptr2->position=this->position-smer*newSize*1.1;
    ptr2->velocity=this->velocity-smer*this->velocity.length()*splitVelocityFactor;
    ptr2->setRadius(newSize);
    world->addLightObject(ptr2);
}
// }}}

// {{{ initialize OK
void Asteroid::initialize(){
    minSpawnRadius=Preferences::_getDouble("/asteroid/minSpawnRadius");
    density=Preferences::_getDouble("/asteroid/density");
    radiusToLive=Preferences::_getDouble("/asteroid/radiusToLive");
    splitRadius=2.2;
    splitVelocityFactor=0.5;

}
// }}}

// {{{ draw OK
void Asteroid::draw(){
    ObjectGraphics::drawAsteroid(this);
}
// }}}
