#include "Engine.h"
#include <SDL/SDL.h>
#include "Errors.h"
#include "Random.h"
#include "Preferences.h"


#define _RESULTS "results.dat"

// {{{ ProbojEngine OK
ProbojEngine::ProbojEngine(){

}
// }}}

// {{{ ~ProbojEngine OK
ProbojEngine::~ProbojEngine(){
}
// }}}

// {{{ initializeGame OK
void ProbojEngine::initializeGame(const std::string& fileName){
    Random::randomize();
  world.initialize(fileName);
  GRAPHICS_DELAY=Preferences::_getDouble("/engine/graphicsDelay");
  SPEED_RATIO=Preferences::_getDouble("/engine/speedRatio");
}
// }}}

//TODO: write player statistics to file
// {{{ runGame OK
void ProbojEngine::runGame(){
    Preferences results;
    double showFPS=0;
    double sendGraphics=0;
    world.initializeServer();
    results.loadFromFile(_RESULTS);

    Errors::_addError("Ready for play",Errors::NOTICE);

    timer.start();

    while (world.getRemainingTime()>0) {

        SDL_Event event;

        while ( SDL_PollEvent(&event) ) {
            if (event.type==SDL_QUIT) return; 
        }

        if (timer.getTimeFromStart()>showFPS) {
            printf("Actual FPS: %5lf  objects %5d\n", timer.getFPS(),
                    world.getObjectCount());
            showFPS+=1;
        }

        if (timer.getTimeFromStart()>sendGraphics){
            sendGraphics+=GRAPHICS_DELAY;
            world.sendDataToViewer();
            world.sendDataToStats();
        }

        world.receiveDataFromPlayers();
        world.computeNextFrame(timer.tick()*SPEED_RATIO);
        world.sendDataToPlayers();
    }
        world.playerStatistics.write();
        world.playerStatistics.save(results,"/results/"+world.mapName);
        results.saveToFile(_RESULTS);
}
// }}}
