#include "Expression.h"
#include "MyStringUtils.h"
#include "Exceptions.h"
#include <cmath>

Expression::Expression():expression("0"){
}

Expression::Expression(const std::string& expression): 
    expression(expression){
}

double Expression::evaluate(double x){
    std::vector<char> var;
    std::vector<double> val;
    var.push_back('x');
    val.push_back(x);
    return MyStringUtils::stringToDouble(compute(expression,var,val));
}

double Expression::evaluate(const std::vector<char>& variables, 
        const std::vector<double>& valuation){
    return MyStringUtils::stringToDouble(compute(expression,variables,valuation));
}


double Expression::functionCall(std::string function, double x){
 if (function=="exp") return exp(x);
 if (function=="log") return log(x);
 if (function=="sqrt") return exp(log(x)/2);
 if (function=="sqr") return x*x;
 throw EIllegalArgument("no function: "+function);
}

std::string Expression::compute(std::string s,
        const std::vector<char>& variables, const std::vector<double>&valuation){
 //   printf("call\n");
    while (true){
   //     printf("%s\n",s.c_str());
        int leading;
        int trailing;
        leading=s.find("[");
        trailing=s.find("]");
        if (trailing<leading) throw EIllegalArgument("expression"
                "mismatch []");
        if (leading!=-1) {
            int pos=leading-1;
            while (pos>=0 && (s[pos]>='a' && s[pos]<='z')) pos--;
            pos++;

            std::string
                inner=s.substr(leading+1,(trailing-leading-1));
            double i=MyStringUtils::stringToDouble(
                    compute(inner,variables,valuation));
            i=functionCall(s.substr(pos,leading-pos),i);

            s.replace(pos, trailing-pos+1,
                    MyStringUtils::doubleToString(i));

            continue;
        }


        leading=s.find("(");
        trailing=s.find(")");
        if (trailing<leading) throw EIllegalArgument("expression"
                "mismatch ()");
        if (leading!=-1) {
            std::string inner=s.substr(leading + 1, trailing - leading
                    - 1);
            s.replace(leading,trailing-leading+1,
                      compute(inner,variables,valuation));
            continue;
        }

        int pos;
        pos=std::max((int)s.rfind('+'),(int)s.rfind('-'));

        if ( pos>0) {
            std::string left,right;

            left=s.substr(0,pos);
            right=s.substr(pos+1,s.size()-pos-1);            

            left=compute(left,variables,valuation);
            right=compute(right,variables,valuation);

            double l=MyStringUtils::stringToDouble(left);
            double r=MyStringUtils::stringToDouble(right);
            if (s[pos]=='+') 
               return MyStringUtils::doubleToString(l+r);
            else 
               return MyStringUtils::doubleToString(l-r);
        }

        pos=std::max((int)s.rfind('*'),(int)s.rfind('/'));

        if ( pos>-1) {
            std::string left,right;

            left=s.substr(0,pos);
            right=s.substr(pos+1,s.size()-pos-1);            

            left=compute(left,variables,valuation);
            right=compute(right,variables,valuation);

            double l=MyStringUtils::stringToDouble(left);
            double r=MyStringUtils::stringToDouble(right);
            if (s[pos]=='*') 
               return MyStringUtils::doubleToString(l*r);
            else 
               return MyStringUtils::doubleToString(l/r);
        }



        if (s.size()==1 && (s[0]>='a' && s[0]<='z')) {
            for (unsigned int q=0;q<variables.size(); q++)
                if (s[0]==variables[q]) return
                     MyStringUtils::doubleToString(valuation[q]);
            throw EIllegalArgument("no variable "+s);

        }
        return s;
    }
}

