#include "Missile.h"
#include "Types.h"
#include "World.h"
#include "Utils.h"
#include "Explosion.h"
#include "Serializer.h"
#include "Preferences.h"
#include "ObjectGraphics.h"

double Missile::detonatingImpact;
double Missile::detonatingDamage;

double Missile::defaultRadius;
double Missile::defaultMass;
double Missile::defaultAngularMass;
double Missile::guessTime;
double Missile::huntDistance;
double Missile::maxAcceleration;
double Missile::maxHuntAcceleration;
double Missile::maxRotationalAcceleration;


// {{{ initialize OK
void Missile::initialize(){
    detonatingImpact=Preferences::_getDouble("/missile/detonatingImpact");
    detonatingDamage=Preferences::_getDouble("/missile/detonatingDamage");
    defaultRadius=Preferences::_getDouble("/missile/radius");
    defaultAngularMass=Preferences::_getDouble("/missile/rotationalMass");
    defaultMass=Preferences::_getDouble("/missile/mass");
    guessTime=Preferences::_getDouble("/missile/guessTime");
    huntDistance=Preferences::_getDouble("/missile/huntDistance");
    maxAcceleration=Preferences::_getDouble("/missile/maxAcceleration");
    maxHuntAcceleration=Preferences::_getDouble("/missile/maxHuntAcceleration");
    maxRotationalAcceleration=Preferences::_getDouble("/missile/maxRotationalAcceleration");
}
// }}}

#define SEARCHING 0
#define GOING 1
#define HUNTING 2

double Missile::get_ship_weight(Ship *target){
    int base=(abs(Utils::getRelativeAngle(this,target))*4.5/M_PI);
    return (double)base+(this->position-target->position).length()/maxlen/10;
}

void Missile::timeNotify(double timeDelta){
    MassObject::timeNotify(timeDelta);
    if(this->state==SEARCHING){
	//VYHLADAVA
	int best=0;
	double value=get_ship_weight(this->world->ships[0]);
	for(unsigned int i=1;i<this->world->ships.size();i++){
	    if(this->world->ships[i]->visible==false) continue;
	    double val=get_ship_weight(this->world->ships[i]);
	    if(val<value){
		best=i;
		value=val;
	    }
	}
	this->target_id=this->world->ships[best]->id;
	this->state=GOING;
    }
    Ship *target=NULL;
    for(unsigned int i=0;i<this->world->ships.size();i++)
	if(this->world->ships[i]->id==this->target_id)target=this->world->ships[i];
    if(target==NULL){
	this->state=SEARCHING;
	return;
    }
    if(this->state==GOING)
	if((this->position-target->position).length()<huntDistance){
	    this->state=HUNTING;
	}
    if(this->state==GOING){
	if(this->lastGuess>guessTime){
	    this->lastGuess=0;
	    this->state=SEARCHING;
	}
    }
    PlayerResponse response;
    if(this->state==HUNTING||this->state==GOING){
	double angle2=Utils::getAngle(this->position,target->position);
	response.rotationJets=Utils::getMassObjectRotationalJets(this,angle2,maxRotationalAcceleration);
	angle2=Utils::getRelativeAngle(this,target);
	if(abs(angle2)<M_PI/9){
	   response.accelerationJets=maxAcceleration;
	   if(this->state==HUNTING)response.accelerationJets=maxHuntAcceleration;
	}
		
    }
    Point direction=Point(cos(angle),sin(angle));
    doImpact(direction*response.accelerationJets*
            timeDelta*maxAcceleration,
            response.rotationJets*timeDelta*
            maxRotationalAcceleration);

    
}

// {{{  getType OK
int Missile::getType(){
    return Types::missile;
}
 // }}}

// {{{ doDamage OK
void Missile::doDamage(double impactDamage, double explosionDamage,  int from, World* world){
    Weapon::doDamage(impactDamage,explosionDamage,from,world);
    double
        pureImpactDamage=impactDamage-explosionDamage*Explosion::impactFactor;
    if (pureImpactDamage>detonatingImpact) this->alive=false; // self kill
}
// }}}

// {{{ destroy OK
void Missile::destroy(World* world){
    if (damage==0) return; // do nothing
    if(this->state==HUNTING){
        Explosion* ptr=world->addExplosion(this->position,damage);
        ptr->owner=this->owner;
        ptr->velocity=velocity;
        ptr->timeNotify(0);
    }
}
// }}}

// {{{ Missile OK 
Missile::Missile(World *world):Weapon(){
    this->world=world;
    this->state=SEARCHING;
    this->lastGuess=0;
    if(world!=NULL)maxlen=sqrt(world->height*world->height+world->width*world->width);else maxlen=1;
    radius=defaultRadius;
    mass=defaultMass;
    angularMass = defaultAngularMass;

    damage=detonatingDamage;
}
// }}}

// {{{ ~Missile OK
Missile::~Missile(){
}
// }}}

// {{{ serialize OK
void Missile::serialize(std::vector<unsigned char>& destination){
    Serializer::serializeChar(destination,getType());
    Weapon::_serialize(destination,this);
    Serializer::serializeFloat(destination,damage);
}
// }}}


// {{{ deserialize OK
Missile* Missile::deserialize(std::vector<unsigned
        char>::iterator& it) {
    Missile* ptr=new Missile(NULL);
    Weapon::_deserialize(it,ptr);
    ptr->damage=Serializer::deserializeFloat(it);
    return ptr;
}
// }}}

// {{{ draw
void Missile::draw(){
    ObjectGraphics::drawMissile(this);
}
// }}}
