#include <iostream>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <errno.h>
#include  <functional>
#include <utility>
#include <algorithm>
#include <vector>
#include <deque>

#include "Proboj.h"
#include "Macro.h"


#include "Errors.h"
#include "Exceptions.h"
#include "Graphics.h"
#include "Preferences.h"
#include "Serializer.h"
#ifndef WIN32
    #include "Font.h"
#endif
#include "events.h"

#include "World.h"
#include <cmath>
#include "Utils.h"
#include "ObjectGraphics.h"
#include "PlayerStatistics.h"

#define FD 47

#define LOG_FILE "viewer.log"
#define INI_FILE "proboj.ini"
#define PERFORMANCE_LOOPS 500
#define MAX_LOOPS 25000
//#define BENCHMARK


int performanceTimer;
int performanceDelta;
int performanceCounter;
int mouseX;
int mouseY;
int frames=0;

int fd;

PlayerStatistics stats;
Surface surface;
PlayerResponse playerResponse;
double angle=0;

std::vector<std::pair<int,int> > Q;

int CONSOLE_COUT=17;
Font font;
int width;
int height;
// {{{ init
int init(){
    playerResponse.fireWeapon=-1;

    Errors::createInstance(LOG_FILE);
    Preferences::createInstance();
    Preferences::_loadFromFile(INI_FILE);

    if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER)<0){
        Errors::_addError("Can't init SDL",Errors::ERROR);
        return 0;
    }
    Errors::_addError("SDL init done",Errors::NOTICE);

   width=Preferences::_getInt("/video/resolution/width");
   height=Preferences::_getInt("/video/resolution/height");
   int bitDepth=Preferences::_getInt("/video/resolution/depth");


   int flags=SDL_SWSURFACE;//|SDL_FULLSCREEN;
    try {
        surface=Surface(SDL_SetVideoMode(width,height,bitDepth,flags),false);
    }catch(Exception &e){
        Errors::_addError(
                std::string("Can't init video mode") +
                e.what(),Errors::ERROR);
        return 0;
    }
    ObjectGraphics::surface=surface;
        
    Errors::_addError("Video mode OK",Errors::NOTICE);
    ObjectGraphics::m.x1=0;
    ObjectGraphics::m.y1=0;
    ObjectGraphics::m.x2=width;
    ObjectGraphics::m.y2=height;
    ObjectGraphics::m.XSCALE=width;
    ObjectGraphics::m.YSCALE=height;
    if (TTF_Init() == -1) {
        Errors::_addError("Unable to initialize SDL_ttf",Errors::ERROR);
        return 0;
    }
    Errors::_addError("Font library OK",Errors::NOTICE);
    font=Font("font.ttf",20); //Font("/var/lib/defoma/x-ttcidfont-conf.d/dirs/TrueType/Times_New_Roman_Italic.ttf",20);
    CONSOLE_COUT=(height-260)/30;
    for(int i=0;i<CONSOLE_COUT*2;i++){Q.push_back(std::make_pair(-1,-1));}
    return 1;

}
// }}}

// {{{ finalize
void finalize(){

    /*
    TTF_Quit();
    Errors::_addError("TTF library closed",Errors::NOTICE);
    */

    SDL_Quit();
    Errors::_addError("SDL closed",Errors::NOTICE);
    Errors::destroyInstance();

}
// }}}

int RRR[]={200,0,0,200,200};
int GGG[]={0,200,0,200,0};
int BBB[]={0,0,200,0,200};


double len=11;
double minlen=10;
 // {{{ main loop doLoop
int doLoop(){
    double timeDelta;

    int now=SDL_GetTicks();
    timeDelta=now-performanceTimer;
    performanceTimer=now;
    performanceDelta+=timeDelta;
    frames++;

    
    
     // {{{ events
    SDL_Event event;
    
    while ( SDL_PollEvent(&event) ) {
        switch (event.type) {
           case SDL_QUIT:
                return 0;
        }
    }
    // }}}
// ------------------------ SEM NESAHAT ---------------------------------------------
    std::vector<unsigned char> tmp;
    tmp.resize(sizeof(int));
    int ret=::recv(fd,&tmp[0],sizeof(int),MSG_PEEK);
    if(ret==-1)return 0;
    int res=(*(int*)&tmp[0]);
    tmp.resize(res);
    unsigned int data=0;
    while(data<tmp.size()){
        ret=::recv(fd,&tmp[data],res-data,0);
	if(ret==-1)return 0;
	data+=ret;
    }
    std::vector<unsigned char>::iterator it=tmp.begin()+sizeof(int);
   // std::cout<<"Dostal som data velkosti "<<tmp.size()<<std::endl;
   // for(unsigned int i=0;i<tmp.size();i++){
//	std::cout<<(int)tmp[i]<<" ";
  //  }std::cout<<std::endl;
 //   assert(tmp[tmp.size()-4]==47&&tmp[tmp.size()-3]==0&&tmp[tmp.size()-2]==0&&tmp[tmp.size()-1]==0,"DOSTAL SOM KREPY PACKET");//TODO -- ukomentit?
    //DESERIALIZE!!!! TODO
//    std::cout<<(int)*it<<std::endl;
    stats.deserialize(it);
    std::vector<unsigned char> events;
    std::vector<std::pair<unsigned char,unsigned char> > kill;
    unsigned int es=Serializer::deserializeInt(it);
    for(unsigned int i=0;i<es;i++)events.push_back(*(it++));
    es=Serializer::deserializeInt(it);
    for(unsigned int i=0;i<es;i++){
	kill.push_back(std::make_pair(*it,*(it+1)));
	it+=2;
    }
    
// ------------------------ KONIEC NESAHANIA ----------------------------------------
    if (++performanceCounter>=PERFORMANCE_LOOPS){
        performanceCounter=0;
        printf("fps: %f, object count %d, serial size %d\n",
                1000*PERFORMANCE_LOOPS/(performanceDelta+0.00001), 
                0,tmp.size());
        performanceDelta=0;

    }
// ------------------------------ tento kod momentalne vykresluje -------------------------------
/*    ObjectGraphics::show=1;
    ObjectGraphics::begin();
    world.draw();
    ObjectGraphics::flip();
    ObjectGraphics::show=0;
    ObjectGraphics::begin();
    world.draw();*/

    for(unsigned int i=0;i<kill.size();i++){
	Q.erase(Q.begin());
	Q.push_back(kill[i]);
    }
    ObjectGraphics::show=1;
//    ObjectGraphics::begin();
    Graphics::filledRectangle(surface,0,0,width-1,height-1,0,0,0,255); 
    double min_score=0;
    double max_score=1;
    for(int i=0;i<stats.playerCount-1;i++){
	min_score=std::min(stats.getScore(i),min_score);
	max_score=std::max(stats.getScore(i),max_score);
    }
    double delta=-min_score+minlen;
    if(delta<minlen)delta=minlen;
    std::vector<std::pair<double,int> > score;
    max_score+=delta;
    min_score+=delta;
#define INCREASE_CONSTANT 0.3
#define INCREASE_CONSTANT_MIN 0.05
    len+=INCREASE_CONSTANT;
    minlen+=INCREASE_CONSTANT_MIN;
    if(len>=width-200){
	len=width-200;
	minlen-=INCREASE_CONSTANT_MIN;
    }
    for(int i=0;i<stats.playerCount-1;i++){
//	std::cout<<stats.getScore(i)<<std::endl;
	score.push_back(std::make_pair(stats.getScore(i)+delta,i));
    }
    std::sort(score.begin(),score.end(),std::greater<std::pair<double,int> >());
    for(unsigned int i=0;i<score.size();i++){
	std::string text=MyStringUtils::intToString(score[i].second+1)+std::string(".");
	Graphics::filledRectangle(surface,30,50+i*40,30+len*score[i].first/max_score,50+i*40+20,RRR[score[i].second],GGG[score[i].second],BBB[score[i].second],255);
#ifndef WIN32
	Graphics::drawText(surface,font,text,10,50+i*40,RRR[score[i].second],GGG[score[i].second],BBB[score[i].second],0);
	text=MyStringUtils::doubleToString(stats.getScore(score[i].second));
	Graphics::drawText(surface,font,text,40+len*score[i].first/max_score,50+i*40,RRR[score[i].second],GGG[score[i].second],BBB[score[i].second],0);
#endif
    }
#ifndef WIN32
    for(unsigned int i=0;i<CONSOLE_COUT;i++){
	std::string text;
	std::string killer,victim="Hrac "+MyStringUtils::intToString(Q[i].second+1);
	switch(Q[i].first){
	    case BLACKHOLE_KILLER: text=victim+" sa rozhodol pristat na ciernej diere.";break;
	    case IMPACT_KILLER: text=victim+" havaroval.";break;
	    case EXPLOSION_KILLER: text=victim+" bol zniceny tlakovou vlnou";break;
	    default: text=victim+" bol rozstrielany hracom "+MyStringUtils::intToString(Q[i].first+1);
	}
	if(Q[i].first>=0)Graphics::drawText(surface,font,text,width/2,260+i*30,50+i*150/CONSOLE_COUT/2,50+i*150/CONSOLE_COUT/2,50+i*150/CONSOLE_COUT/2,0);
    }
    for(unsigned int i=CONSOLE_COUT;i<CONSOLE_COUT*2;i++){
	std::string text;
	std::string killer,victim="Hrac "+MyStringUtils::intToString(Q[i].second+1);
	switch(Q[i].first){
	    case BLACKHOLE_KILLER: text=victim+" sa rozhodol pristat na ciernej diere.";break;
	    case IMPACT_KILLER: text=victim+" havaroval.";break;
	    case EXPLOSION_KILLER: text=victim+" bol zniceny tlakovou vlnou";break;
	    default: text=victim+" bol rozstrielany hracom "+MyStringUtils::intToString(Q[i].first+1);
	}
	if(Q[i].first>=0)Graphics::drawText(surface,font,text,10,260+(i-CONSOLE_COUT)*30,50+i*150/CONSOLE_COUT/2,50+i*150/CONSOLE_COUT/2,50+i*150/CONSOLE_COUT/2,0);
    }
#endif



    ObjectGraphics::flip();
// ----------------------------- koniec vykreslovacieho kodu --------------------------------------

    return 1;

}
// }}}

// {{{ main
int main(int argc, char *argv[]){
    std::string type="local";
    std::string host="0.0.0.0";
    int port=0;
    int i=1;
    while(i<argc){
	    if(!strcmp(argv[i],"-t")){
		    if(i+1>=argc){
			    std::cerr<<"Bat parameters\n";
			    return 1;
		    }
		    type=std::string(argv[i+1]);
		    i+=2;
	    }else if(!strcmp(argv[i],"-h")){
		    if(i+1>=argc){
			    std::cerr<<"Bat parameters\n";
			    return 1;
		    }
		    host=std::string(argv[i+1]);
		    i+=2;
	    }else if(!strcmp(argv[i],"-p")){
		    if(i+1>=argc){
			    std::cerr<<"Bat parameters\n";
			    return 1;
		    }
		    port=atoi(argv[i+1]);
		    i+=2;

	    }else {
		    std::cerr<<"Bat parameters\n";
		    return 1;
	    }
    }
    if(type=="local"){
	    fd=FD;
    }else if(type=="remote"){
	    struct sockaddr_in addr;
	    if (inet_pton(AF_INET, host.c_str(), &addr.sin_addr.s_addr) == -1) {
		    std::cerr<<"inet_pton error: "<<strerror(errno)<<std::endl;
		    return 1;
	    }
	    addr.sin_port = htons(port);
	    addr.sin_family = AF_INET;
	    fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	    if (fd == -1) {
		    std::cerr<<"Nepodarilo sa otvorit socket "<<strerror(errno)<<std::endl;
		    return 1;
	    }
	    if (::connect(fd, (struct sockaddr*)&addr, sizeof(addr))) {
		    std::cerr<<"Nepodaril sa connect "<<strerror(errno)<<std::endl;
		    return 1;
	    }
    }else {
	    std::cerr<<"Bat type\n";
	    return 0;
    }
    std::cout<<"Hrac nastartoval\n";
    try {
        if (init()) {
            while (doLoop());
        }

        finalize();
    } catch (Exception &e){
	shutdown(fd,SHUT_RDWR);
	close(fd);
        printf("There was an exception %s %s\n",
                e.what(),e.getStringStack().c_str());
    } catch (std::exception &e){
	shutdown(fd,SHUT_RDWR);
	close(fd);
        printf("There was an exception %s\n",e.what());
    }
shutdown(fd,SHUT_RDWR);
close(fd);


}

// }}}
