#include <iostream>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <errno.h>

#include "Proboj.h"
#include "Macro.h"


#include "Errors.h"
#include "Exceptions.h"
#include "Graphics.h"
#include "Preferences.h"

#include "World.h"
#include <cmath>
#include "Utils.h"
#include "ObjectGraphics.h"

#define FD 47

#define LOG_FILE "viewer.log"
#define INI_FILE "proboj.ini"
#define PERFORMANCE_LOOPS 500
#define MAX_LOOPS 25000
//#define BENCHMARK


int performanceTimer;
int performanceDelta;
int performanceCounter;
int mouseX;
int mouseY;
int frames=0;

int fd;

World world,tmpWorld;
Surface surface;
PlayerResponse playerResponse;
double angle=0;

// {{{ init
int init(){
    playerResponse.fireWeapon=-1;

    Errors::createInstance(LOG_FILE);
    Preferences::createInstance();
    Preferences::_loadFromFile(INI_FILE);

    if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER)<0){
        Errors::_addError("Can't init SDL",Errors::ERROR);
        return 0;
    }
    Errors::_addError("SDL init done",Errors::NOTICE);

   int width=Preferences::_getInt("/video/resolution/width");
   int height=Preferences::_getInt("/video/resolution/height");
   int bitDepth=Preferences::_getInt("/video/resolution/depth");
    world.initialize("mapa.dat");
    ObjectGraphics::m.x1=0;
    ObjectGraphics::m.x2=width;
    ObjectGraphics::m.y1=height;
    ObjectGraphics::m.y2=0;
    ObjectGraphics::m.XSCALE=world.width;
    ObjectGraphics::m.YSCALE=world.height;

//    tmpWorld.initialize("mapa.dat");

   int flags=SDL_SWSURFACE;
    try {
        surface=Surface(SDL_SetVideoMode(width,height,bitDepth,flags),false);
    }catch(Exception &e){
        Errors::_addError(
                std::string("Can't init video mode") +
                e.what(),Errors::ERROR);
        return 0;
    }
    ObjectGraphics::surface=surface;
        
    Errors::_addError("Video mode OK",Errors::NOTICE);

    /*
    if (TTF_Init() == -1) {
        Errors::_addError("Unable to initialize SDL_ttf",Errors::ERROR);
        return 0;
    }
    Errors::_addError("Font library OK",Errors::NOTICE);
    */


    return 1;

}
// }}}

// {{{ finalize
void finalize(){

    /*
    TTF_Quit();
    Errors::_addError("TTF library closed",Errors::NOTICE);
    */

    SDL_Quit();
    Errors::_addError("SDL closed",Errors::NOTICE);
    Errors::destroyInstance();

}
// }}}


 // {{{ main loop doLoop
int doLoop(){
    double timeDelta;

    int now=SDL_GetTicks();
    timeDelta=now-performanceTimer;
    performanceTimer=now;
    performanceDelta+=timeDelta;
    frames++;

    
    
     // {{{ events
    SDL_Event event;
    
    while ( SDL_PollEvent(&event) ) {
        switch (event.type) {
           case SDL_QUIT:
                return 0;
        }
    }
    // }}}
// ------------------------ SEM NESAHAT ---------------------------------------------
    std::vector<unsigned char> tmp;
    tmp.resize(sizeof(int));
    int ret=::recv(fd,&tmp[0],sizeof(int),MSG_PEEK);
    if(ret==-1)return 0;
    int res=(*(int*)&tmp[0]);
    tmp.resize(res);
    unsigned int data=0;
    while(data<tmp.size()){
        ret=::recv(fd,&tmp[data],res-data,0);
	if(ret==-1)return 0;
	data+=ret;
    }
  //  std::cout<<"--------------------Dostal som data velkosti: "<<tmp.size();
    std::vector<unsigned char>::iterator it=tmp.begin()+sizeof(int);
  //  std::cout<<"-----------------------------\n";
  //  for(int i=0;i<tmp.size();i++){
//	std::cout<<(int)tmp[i]<<" ";
 //   }std::cout<<std::endl;
 //   assert(ret==tmp.size(),"SKUSENE RECV");
    if (tmp.size()==0) return 0;
    assert(tmp[tmp.size()-4]==47&&tmp[tmp.size()-3]==0&&tmp[tmp.size()-2]==0&&tmp[tmp.size()-1]==0,"DOSTAL SOM KREPY PACKET");
    world.deserialize(it);
//    for(unsigned int i=0;i<world.ships.size();i++){
//	assert(world.ships[i]->visible,"TUBAS");
  //  }
    //std::cout<<std::endl;
// ------------------------ KONIEC NESAHANIA ----------------------------------------
    if (++performanceCounter>=PERFORMANCE_LOOPS){
        performanceCounter=0;
        printf("fps: %f, object count %d, serial size %d\n",
                1000*PERFORMANCE_LOOPS/(performanceDelta+0.00001), 
                world.getObjectCount(),tmp.size());
        performanceDelta=0;

    }
// ------------------------------ tento kod momentalne vykresluje -------------------------------
    ObjectGraphics::show=1;
    ObjectGraphics::begin();
    world.draw();
    ObjectGraphics::flip();
    ObjectGraphics::show=0;
    ObjectGraphics::begin();
    world.draw();
// ----------------------------- koniec vykreslovacieho kodu --------------------------------------

    return 1;

}
// }}}

// {{{ main
int main(int argc, char *argv[]){
    std::string type="local";
    std::string host="0.0.0.0";
    int port=0;
    int i=1;
    while(i<argc){
	    if(!strcmp(argv[i],"-t")){
		    if(i+1>=argc){
			    std::cerr<<"Bat parameters\n";
			    return 1;
		    }
		    type=std::string(argv[i+1]);
		    i+=2;
	    }else if(!strcmp(argv[i],"-h")){
		    if(i+1>=argc){
			    std::cerr<<"Bat parameters\n";
			    return 1;
		    }
		    host=std::string(argv[i+1]);
		    i+=2;
	    }else if(!strcmp(argv[i],"-p")){
		    if(i+1>=argc){
			    std::cerr<<"Bat parameters\n";
			    return 1;
		    }
		    port=atoi(argv[i+1]);
		    i+=2;

	    }else {
		    std::cerr<<"Bat parameters\n";
		    return 1;
	    }
    }
    if(type=="local"){
	    fd=FD;
    }else if(type=="remote"){
	    struct sockaddr_in addr;
	    if (inet_pton(AF_INET, host.c_str(), &addr.sin_addr.s_addr) == -1) {
		    std::cerr<<"inet_pton error: "<<strerror(errno)<<std::endl;
		    return 1;
	    }
	    addr.sin_port = htons(port);
	    addr.sin_family = AF_INET;
	    fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	    if (fd == -1) {
		    std::cerr<<"Nepodarilo sa otvorit socket "<<strerror(errno)<<std::endl;
		    return 1;
	    }
	    if (::connect(fd, (struct sockaddr*)&addr, sizeof(addr))) {
		    std::cerr<<"Nepodaril sa connect "<<strerror(errno)<<std::endl;
		    return 1;
	    }
    }else {
	    std::cerr<<"Bat type\n";
	    return 0;
    }
    std::cout<<"Hrac nastartoval\n";
    try {
        if (init()) {
            while (doLoop());
        }

        finalize();
    } catch (Exception &e){
	shutdown(fd,SHUT_RDWR);
	close(fd);
        printf("There was an exception %s %s\n",
                e.what(),e.getStringStack().c_str());
    } catch (std::exception &e){
	shutdown(fd,SHUT_RDWR);
	close(fd);
        printf("There was an exception %s\n",e.what());
    }
shutdown(fd,SHUT_RDWR);
close(fd);


}

// }}}
